
# q-expansion of newform 1200.4.f.x, downloaded from the LMFDB on 10 March 2025.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1200
weight = 4
poly_data = [729, 0, 55, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, 28, 0, 1], 27], [[0, 164, 0, 2], 27], [[110, 0, 4, 0], 1]]

hecke_ring_character_values = [[751, [1, 0, 0, 0]], [901, [1, 0, 0, 0]], [401, [1, 0, 0, 0]], [577, [-1, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0], [0, 3, 0, 0], [0, 0, 0, 0], [0, 1, -1, 0], [8, 0, 0, 3], [0, 41, -2, 0], [0, -4, -5, 0], [-105, 0, 0, -1], [0, -20, -1, 0], [120, 0, 0, 1], [-109, 0, 0, -9], [0, -182, 10, 0], [42, 0, 0, -3], [0, -137, 5, 0], [0, 262, -12, 0], [0, 222, -3, 0], [-542, 0, 0, 8], [387, 0, 0, -4], [0, 105, 37, 0], [-554, 0, 0, -29], [0, 246, -6, 0], [-664, 0, 0, -28], [0, 14, -32, 0], [-712, 0, 0, -4], [0, -1185, 16, 0], [414, 0, 0, -27], [0, -968, -10, 0], [0, 674, -13, 0], [-481, 0, 0, -56], [0, -852, -24, 0], [0, -124, -82, 0], [1634, 0, 0, -25], [0, -442, -6, 0], [-328, 0, 0, 110], [-486, 0, 0, 110], [-747, 0, 0, 23], [0, 97, -148, 0], [0, -2419, 3, 0], [0, 622, 121, 0], [0, -3768, 1, 0], [-1850, 0, 0, -22], [1385, 0, 0, -54], [656, 0, 0, 169], [0, 247, -186, 0], [0, 638, 138, 0], [-413, 0, 0, -73], [809, 0, 0, -179], [0, -2825, 9, 0], [0, -886, -115, 0], [-4461, 0, 0, -92], [0, -910, 185, 0], [-570, 0, 0, -39], [4905, 0, 0, -32], [3824, 0, 0, -94], [0, 4064, -178, 0], [0, 3306, -87, 0], [-4348, 0, 0, 133], [-2388, 0, 0, -30], [0, -1509, 206, 0], [-2936, 0, 0, 57], [0, 973, 81, 0], [0, -4842, -52, 0], [0, -6117, 71, 0], [-1806, 0, 0, 50], [0, 4767, 190, 0], [0, 2716, 82, 0], [-3064, 0, 0, 254], [0, 6747, 82, 0], [0, 3574, 272, 0], [5222, 0, 0, -172], [0, 600, -419, 0], [-7542, 0, 0, 81], [0, -7101, -5, 0], [0, 5827, 138, 0], [2173, 0, 0, -303], [0, -1968, -402, 0], [1866, 0, 0, 57], [0, -2487, -566, 0], [5482, 0, 0, 283], [2223, 0, 0, 166], [-858, 0, 0, 141], [-6062, 0, 0, 322], [-2460, 0, 0, -577], [0, 10039, 74, 0], [7861, 0, 0, 129], [0, -11492, -158, 0], [8856, 0, 0, -450], [0, -3846, -200, 0], [-8302, 0, 0, -169], [0, 2204, 482, 0], [0, 11544, 49, 0], [-4350, 0, 0, -64], [0, 8239, 57, 0], [-13104, 0, 0, -324], [-621, 0, 0, 229], [0, 8794, 49, 0], [3990, 0, 0, 226], [6536, 0, 0, -395], [0, 753, 55, 0], [-3019, 0, 0, 646], [0, 16300, 338, 0], [0, 1262, -440, 0], [0, 912, -611, 0], [-9712, 0, 0, 187], [7177, 0, 0, -223], [0, -7883, -36, 0], [0, 23690, -73, 0], [0, 11046, 327, 0], [8504, 0, 0, 122], [6135, 0, 0, 542], [0, 2792, -352, 0], [0, -15382, -544, 0], [0, 16822, -491, 0], [-2149, 0, 0, 723], [15027, 0, 0, 37], [4992, 0, 0, 114], [0, 12088, -734, 0], [0, -5220, -864, 0], [0, -17958, -104, 0], [8602, 0, 0, 181], [-24126, 0, 0, -24], [0, -1846, 338, 0], [0, -4684, -657, 0], [0, -29512, -40, 0], [900, 0, 0, -946], [4, 0, 0, 527], [-9149, 0, 0, -456], [-16708, 0, 0, -455], [0, 777, 1159, 0], [0, -10930, 452, 0], [3940, 0, 0, 64], [0, 7492, 772, 0], [25292, 0, 0, 314], [0, -15381, -220, 0], [12524, 0, 0, 704], [5641, 0, 0, 1166], [0, 5550, -129, 0], [0, -6369, 1105, 0], [0, -4168, 1814, 0], [-32670, 0, 0, 531], [18589, 0, 0, 1047], [-21650, 0, 0, 900], [0, 6043, -1015, 0], [0, -22758, 507, 0], [22814, 0, 0, -444], [9152, 0, 0, 777], [0, -10225, 0, 0], [0, 12976, 1090, 0], [-13452, 0, 0, 454], [0, 412, -1106, 0], [0, -23647, 528, 0], [-4132, 0, 0, -58], [0, 7263, 689, 0], [0, -19366, -548, 0], [0, -3364, -1108, 0], [-32042, 0, 0, 296], [24767, 0, 0, -693], [-10148, 0, 0, 473], [0, -11, -418, 0], [-7914, 0, 0, -2260], [0, 236, 973, 0], [0, 22096, -788, 0], [0, -1056, 1104, 0], [42470, 0, 0, 34], [0, -27710, -1110, 0], [0, 17836, -1484, 0], [38067, 0, 0, 533], [0, -15114, -1164, 0], [-31917, 0, 0, -124], [0, -3554, -2086, 0], [-1396, 0, 0, 255], [-24053, 0, 0, 1758], [-17398, 0, 0, 718], [0, 13649, -174, 0], [-2081, 0, 0, 2201], [-8492, 0, 0, 178], [-6355, 0, 0, 1863], [-17512, 0, 0, 1437], [0, -8133, 295, 0], [-9391, 0, 0, 1320], [0, 581, -345, 0], [17464, 0, 0, -992], [0, 2313, 2074, 0], [0, -34586, 1064, 0], [0, 5122, 1644, 0], [7634, 0, 0, 1648], [0, -7929, 554, 0], [0, 15973, -1965, 0], [14911, 0, 0, -786], [8552, 0, 0, -57], [0, 4245, -566, 0], [0, -20442, -2217, 0], [-12469, 0, 0, -2531], [12776, 0, 0, 2241], [0, 14108, 1075, 0], [0, 3518, 3216, 0], [48826, 0, 0, 130], [0, 27814, 844, 0], [0, -17574, -1030, 0], [0, -24640, -1907, 0], [10094, 0, 0, -2221], [21024, 0, 0, -782], [0, -11610, -316, 0], [30762, 0, 0, -198], [-50772, 0, 0, 1212], [0, -26928, 1901, 0], [-18005, 0, 0, -1941], [0, -21512, 1511, 0], [-9672, 0, 0, -1950], [-42237, 0, 0, 473], [0, 23693, -624, 0], [54130, 0, 0, -1768], [0, -5549, -555, 0], [0, -25004, 388, 0], [8990, 0, 0, -1208], [1098, 0, 0, -2070], [0, 26565, 785, 0], [1800, 0, 0, -677], [0, -3222, -1251, 0], [0, -9724, 391, 0], [-28459, 0, 0, 414], [26863, 0, 0, 987], [-56682, 0, 0, 1420], [0, -86372, 238, 0], [0, 15302, 1043, 0], [-3083, 0, 0, 2738], [0, 4756, -1759, 0], [-39158, 0, 0, 1010], [0, -22869, -2165, 0], [-2716, 0, 0, 1189], [0, 57237, 290, 0], [19312, 0, 0, 2350], [-28859, 0, 0, -2841], [-74898, 0, 0, 1388], [0, 44432, -742, 0], [0, 9936, -324, 0], [-27742, 0, 0, 2], [0, 58742, 2093, 0], [25846, 0, 0, -96], [-1738, 0, 0, -2767], [0, -14224, 927, 0], [33695, 0, 0, -117], [0, 43317, 3361, 0], [-7467, 0, 0, 2428], [0, -60882, 2116, 0], [96610, 0, 0, -940], [0, 42511, -753, 0], [51908, 0, 0, 1777], [-30227, 0, 0, 255], [0, 48678, -3340, 0], [0, 78858, 396, 0], [-23860, 0, 0, 25], [0, 55176, -30, 0], [-22857, 0, 0, 422], [0, 46632, -1205, 0], [90300, 0, 0, -312], [-9607, 0, 0, -840], [0, -66608, 346, 0], [0, 74126, 2393, 0], [0, 34786, 1048, 0], [0, -56749, 213, 0], [0, 14332, -3127, 0], [-35558, 0, 0, 940], [0, -52925, -3958, 0], [0, -13186, -3836, 0], [34060, 0, 0, 834], [93730, 0, 0, -382], [-91212, 0, 0, 1398], [0, -27460, 248, 0], [0, -20242, 2158, 0], [11011, 0, 0, 3494], [0, -1052, 2118, 0], [0, 34922, 2024, 0], [1041, 0, 0, -5291], [0, 28071, -2848, 0], [0, -55217, -3261, 0], [0, 37586, 1180, 0], [-66587, 0, 0, -366], [9651, 0, 0, -4826], [31188, 0, 0, 1146], [0, 111882, 1353, 0], [112208, 0, 0, -484], [0, -12164, -697, 0], [-5853, 0, 0, -1060], [0, 33067, 3807, 0], [5370, 0, 0, 1444], [0, -28943, 3968, 0], [0, -52314, 3495, 0], [17628, 0, 0, 3498], [37894, 0, 0, 2383], [22054, 0, 0, -810], [0, -752, -1694, 0], [0, -36696, -4458, 0], [86796, 0, 0, -2214], [0, 12939, 1352, 0], [-55290, 0, 0, 2751], [-16437, 0, 0, -853], [0, -27990, 3717, 0], [-97780, 0, 0, 3487], [0, 76436, -1228, 0], [0, -85810, 1532, 0], [0, 71892, -3996, 0], [18805, 0, 0, 1235]]
